from __future__ import annotations

import datetime

LDAP_EPOCH = datetime.datetime(year=1601, month=1, day=1, tzinfo=datetime.timezone.utc)


def from_ldap_date(num_value: int) -> datetime.datetime:
    secs_since_1601 = int(num_value) / 1e7  # original in 100nsec
    return LDAP_EPOCH + datetime.timedelta(seconds=secs_since_1601)


def to_ldap_date(stamp: datetime.datetime) -> int:
    secs_since_1601 = (stamp - LDAP_EPOCH).total_seconds()
    return int(secs_since_1601 * 1e7)  # in 100nsec


LDAP_DATE_MIN = to_ldap_date(datetime.datetime.min.replace(tzinfo=datetime.timezone.utc))
LDAP_DATE_MAX = to_ldap_date(datetime.datetime.max.replace(tzinfo=datetime.timezone.utc))


def parse(value: str) -> datetime.datetime:
    num_value = int(value)
    if num_value >= LDAP_DATE_MAX:
        return datetime.datetime.max
    elif num_value <= LDAP_DATE_MIN:
        return datetime.datetime.min
    return from_ldap_date(num_value)
